==> astk-serveur-1.7.1.1.tbz: aster/ASTK/ASTK_SERV/lib/as_profil.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Definition of ASTER_PROFIL class.
"""

__revision__ = "$Id: as_profil.py 3684 2008-12-15 08:01:04Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import re
import copy
from types import ListType, TupleType
EnumTypes = (ListType, TupleType)

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_PROFIL:
   """Class to read and parse an exported ASTK profile.
   Attributes :
      filename : absolute filename of the '.export' file
      param    : value of each parameters (type dict) (each param is a list)
      args     : arguments (type dict)
      data     : list of the files/directories which are datas
      resu     : list of the files/directories which are results
         (each of data/resu is a dict of : path, ul, compr, type, isrep)
   """
#-------------------------------------------------------------------------------
   def __init__(self, filename=None, run=None):
      """filename : filename of the '.export' file to read
      run   : ASTER_RUN object (optional)
      """
      # initialisations
      self.param = {}
      self.args = {}
      self.data = []
      self.resu = []
      self.filename = filename
      self.verbose = False
      self.debug = False
      # ----- reference to ASTER_RUN object which manages the execution
      self.run = run
      if run != None:
         self.verbose = run['verbose']
         self.debug   = run['debug']
      if filename != None:
         # check if file exists
         if not os.path.isfile(filename):
            self._mess(_('file not found : %s') % filename, _('<A>_ALARM'))
         else:
            # read the file
            f = open(filename, 'r')
            content = f.read()
            f.close()
            self.parse(content)
      
      if self.debug:
         print '<DBG> <init> ASTER_PROFIL :'
         print self

#-------------------------------------------------------------------------------
   def __repr__(self):
      """Pretty print content of the profile
      """
      fmt = ' %-10s = %s'
      fmt2 = ' %-6s %2s %1s %-s %1s'
      txt = []
      txt.append(fmt % ('Filename', self.filename))
      for comm, d in ('Parameters :', self.param), ('Arguments :', self.args):
         txt.append(comm)
         sorted_keys = d.keys()
         sorted_keys.sort()
         for key in sorted_keys:
            txt.append(fmt % (key, d[key]))
      for comm, l in ('Data :', self.data), ('Result :', self.resu):
         txt.append(comm)
         for d in l:
            c = ' '
            if d['compr']:
               c = 'C'
            rep = ' '
            if d['isrep']:
               rep = 'R'
            txt.append(fmt2 % (d['type'], d['ul'], c, d['path'], rep))
      return os.linesep.join(txt)

#-------------------------------------------------------------------------------
   def __getitem__(self, key):
      """Return the value of parameter 'key', or '' if not exists
      (so never raise KeyError exception).
      """
      if self.param.has_key(key):
         return self.param[key]
      else:
         return ['']

#-------------------------------------------------------------------------------
   def Get(self, DR, typ):
      """Return the list of datas (DR='D') or results (DR='R') or both (DR='DR')
      of the type 'typ'
      """
      lr = []
      if   DR == 'D':
         what = self.data
      elif DR == 'R':
         what = self.resu
      else:
         what = self.data + self.resu
      for entry in what:
         if entry['type'] == typ:
            lr.append(entry)
      return lr

#-------------------------------------------------------------------------------
   def Set(self, DR, dico):
      """Add an entry defined in `dico` in .data/.resu.
      """
      if   DR == 'D':
         l_what = [self.data]
      elif DR == 'R':
         l_what = [self.resu]
      else:
         l_what = [self.data, self.resu]
      l_k = ('type', 'isrep', 'path', 'ul', 'compr')
      for k in l_k:
         if not dico.has_key(k):
            self._mess(_("key '%s' missing") % k, _('<F>_PROGRAM_ERROR'))
      for what in l_what:
         what.append(dico)
      # update content
      self.update_content()

#-------------------------------------------------------------------------------
   def Del(self, DR, typ):
      """Delete entries of type 'typ' in .data/.resu.
      """
      lobj = []
      if DR.find('D') > -1:
         for entry in self.data:
            if entry['type'] != typ:
               lobj.append(entry)
         self.data = lobj
      lobj = []
      if DR.find('R') > -1:
         for entry in self.resu:
            if entry['type'] != typ:
               lobj.append(entry)
         self.resu = lobj

#-------------------------------------------------------------------------------
   def __setitem__(self, p, v):
      """Add the parameter 'p' with the value 'v'
      """
      if self.debug and self.param.has_key(p):
         print '<DBG> (AS_PROFIL.setitem) force param['+p+'] = ', v
      if type(v) not in EnumTypes:
         v = [v,]
      self.param[p] = v

#-------------------------------------------------------------------------------
   def __delitem__(self, p):
      """Delete the parameter 'p'
      """
      if self.param.has_key(p):
         del self.param[p]

#-------------------------------------------------------------------------------
   def _mess(self, msg, cod='', store=False):
      """Just print a message
      """
      if hasattr(self.run, 'Mess'):
         self.run.Mess(msg, cod, store)
      else:
         print '%-18s %s' % (cod, msg)

#-------------------------------------------------------------------------------
   def parse(self, content):
      """Extract fields of config from 'content'
      """
      for l in content.split(os.linesep):
         if not re.search('^[ ]*#', l):
            spl = l.split()
            typ = ''
            if len(spl) > 0:
               typ = spl[0]
               if typ not in ('A', 'P', 'F', 'R'):
                  self._mess(_('unexpected type : %s') % typ, _('<A>_ALARM'))
            else:
               continue
            if len(spl) >= 3:
               if typ == 'P':
                  if self.param.has_key(spl[1]):
                     self.param[spl[1]].append(' '.join(spl[2:]))
                  else:
                     self.param[spl[1]] = [' '.join(spl[2:])]
               elif typ == 'A':
                  self.args[spl[1]] = ' '.join(spl[2:])
               elif typ == 'F' or typ == 'R':
                  if len(spl) >= 5:
                     dico = {
                        'type'  : spl[1],
                        'path'  : spl[2],
                        'isrep' : typ == 'R',
                        'ul'    : spl[4],
                        'compr' : spl[3].find('C') > -1,
                     }
                     if spl[3].find('D')>-1:
                        self.data.append(dico)
                     if spl[3].find('R')>-1:
                        self.resu.append(dico)
                  else:
                     self._mess(_('fields missing on line : %s') % l, _('<A>_ALARM'))
            elif len(spl) >= 2:
               typ = spl[0]
               if typ == 'A':
                  self.args[spl[1]] = ''
      self.update_content()

#-------------------------------------------------------------------------------
   def copy(self):
      """Return a copy of the profile
      """
      newp = self.__class__(None, self.run)
      for attr in ('args', 'param'):
         setattr(newp, attr, getattr(self, attr).copy())
      for attr in ('data', 'resu'):
         for dico in getattr(self, attr):
            getattr(newp, attr).append(dico.copy())
      return newp
      
#-------------------------------------------------------------------------------
   def update(self, other):
      """Update the profile using values from 'other' :
            - replace params and args
            - add datas and results
      """
      self.param.update(other.param)
      self.args.update(other.args)
      self.data.extend(other.data)
      self.resu.extend(other.resu)

#-------------------------------------------------------------------------------
   def update_content(self, dbg=False):
      """Fill 'content' attribute.
      """
      txt = []
      for p, dico in ('P', self.param), ('A', self.args):
         sorted_keys = dico.keys()
         sorted_keys.sort()
         for key in sorted_keys:
            l_val = dico[key]
            if type(l_val) not in EnumTypes:
               l_val = [l_val,]
            for v in l_val:
               txt.append(' '.join([p, key, str(v)]))
      for dr, l_val in ('D', self.data), ('R', self.resu):
         for d in l_val:
            c = ' '
            if d['compr']:
               c = 'C'
            fr = 'F'
            if d['isrep']:
               fr = 'R'
            txt.append(' '.join([fr, d['type'], d['path'], dr+c, str(d['ul'])]))
      txt.append('')
      self.content = os.linesep.join(txt)

#-------------------------------------------------------------------------------
   def WriteExportTo(self, fich, dbg=False):
      """Write the export file represents this profile.
      """
      self.filename = self.filename or fich
      self.update_content()
      if self.debug or dbg:
         print '<DBG> <-- content of "%s"' % fich
         print self.content
         print '<DBG> end of file -->'
      try:
         open(fich, 'w').write(self.content)
      except IOError, msg:
         self._mess(_('No write access to %s') % fich, _('<F>_ERROR'))

#-------------------------------------------------------------------------------
if __name__ == '__main__':
   prof = ASTER_PROFIL('../unittest/etude/etude.export')
   print prof

==> astk-serveur-1.7.4.1.tbz: aster/ASTK/ASTK_SERV/lib/as_profil.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Definition of ASTER_PROFIL class.
"""

__revision__ = "$Id: as_profil.py 3897 2009-05-28 15:39:58Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import re
import copy
from utils import get_absolute_path

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_PROFIL:
   """Class to read and parse an exported ASTK profile.
   Attributes :
      filename : absolute filename of the '.export' file
      param    : value of each parameters (type dict) (each param is a list)
      args     : arguments (type dict)
      data     : list of the files/directories which are datas
      resu     : list of the files/directories which are results
         (each of data/resu is a dict of : path, ul, compr, type, isrep)
   """
#-------------------------------------------------------------------------------
   def __init__(self, filename=None, run=None):
      """filename : filename of the '.export' file to read
      run   : ASTER_RUN object (optional)
      """
      # initialisations
      self.param = {}
      self.args = {}
      self.data = []
      self.resu = []
      self.agla = []
      self.filename = filename
      self.verbose = False
      self.debug = False
      self.content = None
      # ----- reference to ASTER_RUN object which manages the execution
      self.run = run
      if run != None:
         self.verbose = run['verbose']
         self.debug   = run['debug']
      if filename != None:
         # check if file exists
         if not os.path.isfile(filename):
            self._mess(_('file not found : %s') % filename, _('<A>_ALARM'))
         else:
            # read the file
            f = open(filename, 'r')
            content = f.read()
            f.close()
            self.parse(content)
      
      if self.debug:
         print '<DBG> <init> ASTER_PROFIL :'
         print self

#-------------------------------------------------------------------------------
   def __repr__(self):
      """Pretty print content of the profile
      """
      fmt = ' %-10s = %s'
      fmt2 = ' %-6s %2s %1s %-s %1s'
      txt = []
      txt.append(fmt % ('Filename', self.filename))
      for comm, d in ('Parameters :', self.param), ('Arguments :', self.args):
         txt.append(comm)
         sorted_keys = d.keys()
         sorted_keys.sort()
         for key in sorted_keys:
            txt.append(fmt % (key, d[key]))
      for comm, l in ('Data :', self.data), ('Result :', self.resu):
         txt.append(comm)
         for d in l:
            c = ' '
            if d['compr']:
               c = 'C'
            rep = ' '
            if d['isrep']:
               rep = 'R'
            txt.append(fmt2 % (d['type'], d['ul'], c, d['path'], rep))
      txt.append('Agla fields :')
      for val in self.agla:
         txt.append(' '.join(val))
      return os.linesep.join(txt)

#-------------------------------------------------------------------------------
   def __getitem__(self, key):
      """Return the value of parameter 'key', or '' if not exists
      (so never raise KeyError exception).
      """
      if self.param.has_key(key):
         return self.param[key]
      else:
         return ['']

#-------------------------------------------------------------------------------
   def Get(self, DR, typ):
      """Return the list of datas (DR='D') or results (DR='R') or both (DR='DR')
      of the type 'typ'
      """
      lr = []
      if   DR == 'D':
         what = self.data
      elif DR == 'R':
         what = self.resu
      else:
         what = self.data + self.resu
      for entry in what:
         if entry['type'] == typ:
            lr.append(entry)
      return lr

#-------------------------------------------------------------------------------
   def Set(self, DR, dico):
      """Add an entry defined in `dico` in .data/.resu.
      """
      if   DR == 'D':
         l_what = [self.data]
      elif DR == 'R':
         l_what = [self.resu]
      else:
         l_what = [self.data, self.resu]
      l_k = ('type', 'isrep', 'path', 'ul', 'compr')
      for k in l_k:
         if not dico.has_key(k):
            self._mess(_("key '%s' missing") % k, _('<F>_PROGRAM_ERROR'))
      for what in l_what:
         what.append(dico)
      # update content
      self.update_content()

#-------------------------------------------------------------------------------
   def Del(self, DR, typ):
      """Delete entries of type 'typ' in .data/.resu.
      """
      lobj = []
      if DR.find('D') > -1:
         for entry in self.data:
            if entry['type'] != typ:
               lobj.append(entry)
         self.data = lobj
      lobj = []
      if DR.find('R') > -1:
         for entry in self.resu:
            if entry['type'] != typ:
               lobj.append(entry)
         self.resu = lobj

#-------------------------------------------------------------------------------
   def __setitem__(self, p, v):
      """Add the parameter 'p' with the value 'v'
      """
      if self.debug and self.param.has_key(p):
         print '<DBG> (AS_PROFIL.setitem) force param['+p+'] = ', v
      if type(v) not in (list, tuple):
         v = [v,]
      self.param[p] = v

#-------------------------------------------------------------------------------
   def __delitem__(self, p):
      """Delete the parameter 'p'
      """
      if self.param.has_key(p):
         del self.param[p]

#-------------------------------------------------------------------------------
   def _mess(self, msg, cod='', store=False):
      """Just print a message
      """
      if hasattr(self.run, 'Mess'):
         self.run.Mess(msg, cod, store)
      else:
         print '%-18s %s' % (cod, msg)

#-------------------------------------------------------------------------------
   def parse(self, content):
      """Extract fields of config from 'content'
      """
      for l in content.split(os.linesep):
         if not re.search('^[ ]*#', l):
            spl = l.split()
            typ = ''
            if len(spl) > 0:
               typ = spl[0]
               if typ not in ('A', 'P', 'F', 'R', 'N'):
                  self._mess(_('unexpected type : %s') % typ, _('<A>_ALARM'))
            else:
               continue
            if len(spl) >= 3:
               if typ == 'P':
                  if self.param.has_key(spl[1]):
                     self.param[spl[1]].append(' '.join(spl[2:]))
                  else:
                     self.param[spl[1]] = [' '.join(spl[2:])]
               elif typ == 'A':
                  self.args[spl[1]] = ' '.join(spl[2:])
               elif typ == 'F' or typ == 'R':
                  if len(spl) >= 5:
                     dico = {
                        'type'  : spl[1],
                        'path'  : spl[2],
                        'isrep' : typ == 'R',
                        'ul'    : spl[4],
                        'compr' : spl[3].find('C') > -1,
                     }
                     if spl[3].find('D')>-1:
                        self.data.append(dico)
                     if spl[3].find('R')>-1:
                        self.resu.append(dico)
                  else:
                     self._mess(_('fields missing on line : %s') % l, _('<A>_ALARM'))
               elif typ == 'N':
                  # just store agla fields
                  self.agla.append(spl)
            elif len(spl) >= 2:
               typ = spl[0]
               if typ == 'A':
                  self.args[spl[1]] = ''
      self.update_content()

#-------------------------------------------------------------------------------
   def copy(self):
      """Return a copy of the profile
      """
      newp = self.__class__(None, self.run)
      for attr in ('filename', 'content'):
         setattr(newp, attr, getattr(self, attr))
      for attr in ('args', 'param'):
         setattr(newp, attr, getattr(self, attr).copy())
      for attr in ('data', 'resu'):
         for dico in getattr(self, attr):
            getattr(newp, attr).append(dico.copy())
      for val in self.agla:
         newp.append(val[:])
      return newp
      
#-------------------------------------------------------------------------------
   def update(self, other):
      """Update the profile using values from 'other' :
            - replace params and args
            - add datas and results
      """
      self.param.update(other.param)
      self.args.update(other.args)
      self.data.extend(other.data)
      self.resu.extend(other.resu)
      self.agla.extend(other.agla)

#-------------------------------------------------------------------------------
   def update_content(self, dbg=False):
      """Fill 'content' attribute.
      """
      txt = []
      for p, dico in ('P', self.param), ('A', self.args):
         sorted_keys = dico.keys()
         sorted_keys.sort()
         for key in sorted_keys:
            l_val = dico[key]
            if type(l_val) not in (list, tuple):
               l_val = [l_val,]
            for v in l_val:
               txt.append(' '.join([p, key, str(v)]))
      for dr, l_val in ('D', self.data), ('R', self.resu):
         for d in l_val:
            c = ' '
            if d['compr']:
               c = 'C'
            fr = 'F'
            if d['isrep']:
               fr = 'R'
            txt.append(' '.join([fr, d['type'], d['path'], dr+c, str(d['ul'])]))
      for val in self.agla:
         txt.append(' '.join(val))
      txt.append('')
      self.content = os.linesep.join(txt)

#-------------------------------------------------------------------------------
   def WriteExportTo(self, fich, dbg=False):
      """Write the export file represents this profile.
      """
      self.filename = self.filename or fich
      self.update_content()
      if self.debug or dbg:
         print '<DBG> <-- content of "%s"' % fich
         print self.content
         print '<DBG> end of file -->'
      try:
         open(fich, 'w').write(self.content)
      except IOError, msg:
         self._mess(_('No write access to %s') % fich, _('<F>_ERROR'))
      except Exception, msg:
         self._mess(_('Can not write export file : %s') % fich, _('<F>_ERROR'))

#-------------------------------------------------------------------------------
if __name__ == '__main__':
   prof = ASTER_PROFIL('../unittest/etude/etude.export')
   print prof
